@@ -29,6 +29,7 @@
 #include <QTimer>
 #include <QDBusInterface>
 #include <QDBusPendingCall>
+#include <QCloseEvent>
 
 // include files for KDE
 #include <kxmlgui_version.h>
@@ -37,7 +38,8 @@
 #include <klocalizedstring.h>
 #include <kstandardaction.h>
 #include <kxmlguifactory.h>
-#include <KProcess>
+#include <kprocess.h>
+#include <kcoreaddons_version.h>
 
 // KMix
 #include "kmix_debug.h"
@@ -60,19 +62,24 @@
  * Constructs a mixer window (KMix main window)
  */
 
-KMixWindow::KMixWindow(bool invisible, bool reset) :
-		KXmlGuiWindow(nullptr, Qt::WindowFlags(KDE_DEFAULT_WINDOWFLAGS|Qt::WindowContextHelpButtonHint)),
-		m_autouseMultimediaKeys(true),
-		m_dockWidget(), m_dsm(0), m_dontSetDefaultCardOnStart(false)
+KMixWindow::KMixWindow(KMixApp::StartupOptions options)
+        : KXmlGuiWindow(nullptr),
+	  m_autouseMultimediaKeys(true),
+	  m_dockWidget(nullptr),
+	  m_dontSetDefaultCardOnStart(false)
 {
 	setObjectName(QStringLiteral("KMixWindow"));
 	// disable delete-on-close because KMix might just sit in the background waiting for cards to be plugged in
 	setAttribute(Qt::WA_DeleteOnClose, false);
 
+	m_noDockWidget = (options & KMixApp::NoSystemTray);
+	m_startVisible = true;				// force on for failsafe reset
+
+	const bool reset = (options & KMixApp::FailsafeReset);
 	initActions(); // init actions first, so we can use them in the loadConfig() already
-	loadAndInitConfig(reset); // Load config before initMixer(), e.g. due to "MultiDriver" keyword
+	if (!reset) loadBaseConfig(); // Load config before initMixer(), e.g. due to "MultiDriver" keyword
 	initActionsLate(); // init actions that require a loaded config
-	// TODO: Port to KF5
+	// TODO: Port to KF5, see MixerBackend::translateKernelToWhatsthis()
 	//KGlobal::locale()->insertCatalog(QLatin1String("kmix-controls"));
 	initWidgets();
 	initPrefDlg();
@@ -92,9 +99,9 @@ KMixWindow::KMixWindow(bool invisible, bool reset) :
 
 	fixConfigAfterRead();
 
-	if (m_startVisible && !invisible) show();	// Started visible
+	if (m_startVisible && !(options & KMixApp::KeepVisibility)) show();	// Started visible
 
-	connect(qApp, SIGNAL(aboutToQuit()), SLOT(saveConfig()) );
+	connect(qApp, &QCoreApplication::aboutToQuit, this, &KMixWindow::saveConfig);
 
 	ControlManager::instance()->addListener(
 			QString(), // All mixers (as the Global master Mixer might change)
@@ -111,8 +118,6 @@ KMixWindow::~KMixWindow()
 {
 	ControlManager::instance()->removeListener(this);
 
-	delete m_dsm;
-
 	// -1- Cleanup Memory: clearMixerWidgets
 	while (m_wsMixers->count() != 0)
 	{
@@ -133,9 +138,9 @@ KMixWindow::~KMixWindow()
 	// Until we do so, this is the best place to call clearCache(). Later, e.g. in main() would likely be problematic.
 
 	GUIProfile::clearCache();
-
 }
 
+
 void KMixWindow::controlsChange(ControlManager::ChangeType changeType)
 {
 	switch (changeType)
@@ -149,77 +154,78 @@ void KMixWindow::controlsChange(ControlManager::ChangeType changeType)
 		ControlManager::warnUnexpectedChangeType(changeType, this);
 		break;
 	}
-
 }
 
 
 void KMixWindow::initActions()
 {
 	// file menu
-	KStandardAction::quit(this, SLOT(quit()), actionCollection());
+	KStandardAction::quit(this, &QCoreApplication::quit, actionCollection());
+
+	QAction *action = actionCollection()->addAction(QStringLiteral("hide_kmixwindow"));
+	action->setText(i18n("Hide Mixer Window"));
+	connect(action, &QAction::triggered, this, &QWidget::close);
+	actionCollection()->setDefaultShortcut(action, Qt::Key_Escape);
 
 	// settings menu
-	_actionShowMenubar = KStandardAction::showMenubar(this, SLOT(toggleMenuBar()), actionCollection());
-	KStandardAction::preferences(this, SLOT(showSettings()), actionCollection());
-
+	_actionShowMenubar = KStandardAction::showMenubar(this, &KMixWindow::toggleMenuBar, actionCollection());
+	KStandardAction::preferences(this, &KMixWindow::showSettings, actionCollection());
 	KStandardAction::keyBindings(guiFactory(), &KXMLGUIFactory::showConfigureShortcutsDialog, actionCollection());
 
-	QAction* action = actionCollection()->addAction(QStringLiteral("launch_kdesoundsetup"));
+	action = actionCollection()->addAction(QStringLiteral("launch_kdesoundsetup"));
 	action->setText(i18n("Audio Setup..."));
-	connect(action, SIGNAL(triggered(bool)), SLOT(slotKdeAudioSetupExec()));
-
-	action = actionCollection()->addAction(QStringLiteral("hide_kmixwindow"));
-	action->setText(i18n("Hide Mixer Window"));
-	connect(action, SIGNAL(triggered(bool)), SLOT(hideOrClose()));
-	actionCollection()->setDefaultShortcut(action, Qt::Key_Escape);
+	action->setIcon(QIcon::fromTheme("speaker"));
+	connect(action, &QAction::triggered, this, &KMixWindow::slotKdeAudioSetupExec);
 
 	action = actionCollection()->addAction(QStringLiteral("toggle_channels_currentview"));
 	action->setText(i18n("Configure &Channels..."));
-	connect(action, SIGNAL(triggered(bool)), SLOT(slotConfigureCurrentView()));
+	action->setIcon(QIcon::fromTheme("settings-channels"));
+	connect(action, &QAction::triggered, this, &KMixWindow::slotConfigureCurrentView);
 
 	action = actionCollection()->addAction(QStringLiteral("select_master"));
 	action->setText(i18n("Select Master Channel..."));
-	connect(action, SIGNAL(triggered(bool)), SLOT(slotSelectMaster()));
+	action->setIcon(QIcon::fromTheme("settings-master"));
+	connect(action, &QAction::triggered, this, &KMixWindow::slotSelectMaster);
 
 	action = actionCollection()->addAction(QStringLiteral("save_1"));
-	actionCollection()->setDefaultShortcut(action, Qt::CTRL + Qt::SHIFT + Qt::Key_1);
+	actionCollection()->setDefaultShortcut(action, Qt::CTRL|Qt::SHIFT|Qt::Key_1);
 	action->setText(i18n("Save volume profile 1"));
-	connect(action, SIGNAL(triggered(bool)), SLOT(saveVolumes1()));
+	connect(action, &QAction::triggered, this, &KMixWindow::saveVolumes1);
 
 	action = actionCollection()->addAction(QStringLiteral("save_2"));
-	actionCollection()->setDefaultShortcut(action, Qt::CTRL + Qt::SHIFT + Qt::Key_2);
+	actionCollection()->setDefaultShortcut(action, Qt::CTRL|Qt::SHIFT|Qt::Key_2);
 	action->setText(i18n("Save volume profile 2"));
-	connect(action, SIGNAL(triggered(bool)), SLOT(saveVolumes2()));
+	connect(action, &QAction::triggered, this, &KMixWindow::saveVolumes2);
 
 	action = actionCollection()->addAction(QStringLiteral("save_3"));
-	actionCollection()->setDefaultShortcut(action, Qt::CTRL + Qt::SHIFT + Qt::Key_3);
+	actionCollection()->setDefaultShortcut(action, Qt::CTRL|Qt::SHIFT|Qt::Key_3);
 	action->setText(i18n("Save volume profile 3"));
-	connect(action, SIGNAL(triggered(bool)), SLOT(saveVolumes3()));
+	connect(action, &QAction::triggered, this, &KMixWindow::saveVolumes3);
 
 	action = actionCollection()->addAction(QStringLiteral("save_4"));
-	actionCollection()->setDefaultShortcut(action, Qt::CTRL + Qt::SHIFT + Qt::Key_4);
+	actionCollection()->setDefaultShortcut(action, Qt::CTRL|Qt::SHIFT|Qt::Key_4);
 	action->setText(i18n("Save volume profile 4"));
-	connect(action, SIGNAL(triggered(bool)), SLOT(saveVolumes4()));
+	connect(action, &QAction::triggered, this, &KMixWindow::saveVolumes4);
 
 	action = actionCollection()->addAction(QStringLiteral("load_1"));
-	actionCollection()->setDefaultShortcut(action, Qt::CTRL + Qt::Key_1);
+	actionCollection()->setDefaultShortcut(action, Qt::CTRL|Qt::Key_1);
 	action->setText(i18n("Load volume profile 1"));
-	connect(action, SIGNAL(triggered(bool)), SLOT(loadVolumes1()));
+	connect(action, &QAction::triggered, this, &KMixWindow::loadVolumes1);
 
 	action = actionCollection()->addAction(QStringLiteral("load_2"));
-	actionCollection()->setDefaultShortcut(action, Qt::CTRL + Qt::Key_2);
+	actionCollection()->setDefaultShortcut(action, Qt::CTRL|Qt::Key_2);
 	action->setText(i18n("Load volume profile 2"));
-	connect(action, SIGNAL(triggered(bool)), SLOT(loadVolumes2()));
+	connect(action, &QAction::triggered, this, &KMixWindow::loadVolumes2);
 
 	action = actionCollection()->addAction(QStringLiteral("load_3"));
-	actionCollection()->setDefaultShortcut(action, Qt::CTRL + Qt::Key_3);
+	actionCollection()->setDefaultShortcut(action, Qt::CTRL|Qt::Key_3);
 	action->setText(i18n("Load volume profile 3"));
-	connect(action, SIGNAL(triggered(bool)), SLOT(loadVolumes3()));
+	connect(action, &QAction::triggered, this, &KMixWindow::loadVolumes3);
 
 	action = actionCollection()->addAction(QStringLiteral("load_4"));
-	actionCollection()->setDefaultShortcut(action, Qt::CTRL + Qt::Key_4);
+	actionCollection()->setDefaultShortcut(action, Qt::CTRL|Qt::Key_4);
 	action->setText(i18n("Load volume profile 4"));
-	connect(action, SIGNAL(triggered(bool)), SLOT(loadVolumes4()));
+	connect(action, &QAction::triggered, this, &KMixWindow::loadVolumes4);
 
 	createGUI(QLatin1String("kmixui.rc"));
 }
@@ -233,17 +239,17 @@ void KMixWindow::initActionsLate()
 
 		KGlobalAccel::setGlobalShortcut(globalAction, Qt::Key_VolumeUp);
 
-		connect(globalAction, SIGNAL(triggered(bool)), SLOT(slotIncreaseVolume()));
+		connect(globalAction, &QAction::triggered, this, &KMixWindow::slotIncreaseVolume);
 
 		globalAction = actionCollection()->addAction(QStringLiteral("decrease_volume"));
 		globalAction->setText(i18n("Decrease Volume"));
 		KGlobalAccel::setGlobalShortcut(globalAction, Qt::Key_VolumeDown);
-		connect(globalAction, SIGNAL(triggered(bool)), SLOT(slotDecreaseVolume()));
+		connect(globalAction, &QAction::triggered, this, &KMixWindow::slotDecreaseVolume);
 
 		globalAction = actionCollection()->addAction(QStringLiteral("mute"));
 		globalAction->setText(i18n("Mute"));
 		KGlobalAccel::setGlobalShortcut(globalAction, Qt::Key_VolumeMute);
-		connect(globalAction, SIGNAL(triggered(bool)), SLOT(slotMute()));
+		connect(globalAction, &QAction::triggered, this, &KMixWindow::slotMute);
 	}
 }
 
@@ -257,7 +263,7 @@ void KMixWindow::initActionsAfterInitMixer()
 		_cornerLabelNew->setIcon(QIcon::fromTheme("tab-new"));
 		_cornerLabelNew->setToolTip(i18n("Add new view"));
 		m_wsMixers->setCornerWidget(_cornerLabelNew, Qt::TopLeftCorner);
-		connect(_cornerLabelNew, SIGNAL(clicked()), SLOT(newView()));
+		connect(_cornerLabelNew, &QAbstractButton::clicked, this, &KMixWindow::newView);
 	}
 
 	// If PulseAudio is in use, then that handles device hotplugging itself.
@@ -292,12 +298,13 @@ void KMixWindow::initWidgets()
 	m_wsMixers->setDocumentMode(true);
 	setCentralWidget(m_wsMixers);
 	m_wsMixers->setTabsClosable(false);
-	connect(m_wsMixers, SIGNAL(tabCloseRequested(int)), SLOT(saveAndCloseView(int)));
-
-	connect(m_wsMixers, SIGNAL(currentChanged(int)), SLOT(newMixerShown(int)));
+	connect(m_wsMixers, &QTabWidget::tabCloseRequested, this, &KMixWindow::saveAndCloseView);
+	// TODO: connect this signal after all tabs are added, then there
+	// will be no need for m_dontSetDefaultCardOnStart
+	connect(m_wsMixers, &QTabWidget::currentChanged, this, &KMixWindow::newMixerShown);
 
 	// show menubar if the actions says so (or if the action does not exist)
-	menuBar()->setVisible((_actionShowMenubar == 0) || _actionShowMenubar->isChecked());
+	menuBar()->setVisible((_actionShowMenubar==nullptr) || _actionShowMenubar->isChecked());
 }
 
 void KMixWindow::setInitialSize()
@@ -316,15 +323,20 @@ void KMixWindow::setInitialSize()
 	if (!pos.isNull()) move(pos);
 }
 
+
 void KMixWindow::removeDock()
 {
-	if (m_dockWidget)
-	{
-		m_dockWidget->deleteLater();
-		m_dockWidget = 0;
-	}
+	if (m_dockWidget!=nullptr) m_dockWidget->deleteLater();
+	m_dockWidget = nullptr;
 }
 
+
+bool KMixWindow::shouldShowDock() const
+{
+	return (!m_noDockWidget && Settings::showDockWidget());
+}
+
+
 /**
  * Creates or deletes the KMixDockWidget, depending on whether there is a Mixer instance available.
  *
@@ -333,17 +345,23 @@ void KMixWindow::removeDock()
  */
 bool KMixWindow::updateDocking()
 {
-	if (!Settings::showDockWidget() || MixerToolBox::mixers().isEmpty())
+	const bool showDock = shouldShowDock();
+
+	// If there is no system tray icon, then disable this action
+	// to indicate that closing the window will also quit KMix.
+	QAction *act = actionCollection()->action(QStringLiteral("hide_kmixwindow"));
+	if (act!=nullptr) act->setEnabled(showDock);
+
+	if (!showDock) removeDock();
+	else
 	{
-		removeDock();
-		return false;
+		if (m_dockWidget==nullptr) m_dockWidget = new KMixDockWidget(this);
 	}
-	if (!m_dockWidget)
-	{
-		m_dockWidget = new KMixDockWidget(this);
-	}
-	return true;
+
+	return (showDock);
 }
+
+
 void KMixWindow::saveConfig()
 {
 	saveBaseConfig();
@@ -441,7 +459,7 @@ void KMixWindow::saveVolumes(const QString &postfix)
 	const QString& kmixctrlRcFilename = getKmixctrlRcFilename(postfix);
 	KConfig cfg(kmixctrlRcFilename);
 
-	for (const Mixer *mixer : qAsConst(MixerToolBox::mixers()))
+	for (const Mixer *mixer : std::as_const(MixerToolBox::mixers()))
 	{
 		// Protect from unplugged devices - better to *not* save them
 		if (mixer->isOpen()) mixer->volumeSave(&cfg);
@@ -463,12 +481,6 @@ QString KMixWindow::getKmixctrlRcFilename(const QString &postfix)
 }
 
 
-void KMixWindow::loadAndInitConfig(bool reset)
-{
-	if (!reset) loadBaseConfig();
-}
-
-
 void KMixWindow::loadBaseConfig()
 {
 	m_startVisible = Settings::visible();
@@ -499,7 +511,7 @@ void KMixWindow::loadVolumes(const QString &postfix)
 	const QString &kmixctrlRcFilename = getKmixctrlRcFilename(postfix);
 	const KConfig cfg(kmixctrlRcFilename);
 
-	for (Mixer *mixer : qAsConst(MixerToolBox::mixers()))
+	for (Mixer *mixer : std::as_const(MixerToolBox::mixers()))
 	{
 		mixer->volumeLoad(&cfg);
 	}
@@ -516,6 +528,7 @@ void KMixWindow::recreateGUI(bool saveConfig, bool reset)
 	recreateGUI(saveConfig, QString(), false, reset);
 }
 
+
 /**
  * Create or recreate the Mixer GUI elements
  *
@@ -538,11 +551,8 @@ void KMixWindow::recreateGUI(bool saveConfig, const QString& mixerId, bool force
 	QList<GUIProfile*> activeGuiProfiles;
 	for (int i = 0; i < m_wsMixers->count(); ++i)
 	{
-		KMixerWidget* kmw = dynamic_cast<KMixerWidget*>(m_wsMixers->widget(i));
-		if (kmw)
-		{
-			activeGuiProfiles.append(kmw->getGuiprof());
-		}
+		const KMixerWidget *kmw = qobject_cast<const KMixerWidget *>(m_wsMixers->widget(i));
+		if (kmw!=nullptr) activeGuiProfiles.append(kmw->getGuiprof());
 	}
 
 	for (const GUIProfile *guiprof : std::as_const(activeGuiProfiles))
@@ -556,7 +566,7 @@ void KMixWindow::recreateGUI(bool saveConfig, const QString& mixerId, bool force
 		mixerHasProfile[mixer] = true;
 
 		KMixerWidget* kmw = findKMWforTab(guiprof->getId());
-		if ( kmw == 0 )
+		if (kmw==nullptr)
 		{
 			// does not yet exist => create
 			addMixerWidget(mixer->id(), guiprof->getId(), -1);
@@ -587,13 +597,13 @@ void KMixWindow::recreateGUI(bool saveConfig, const QString& mixerId, bool force
 		// No TAB YET => This should mean KMix is just started, or the user has just plugged in a card
 
 		{
-			GUIProfile* guiprof = 0;
+			GUIProfile *guiprof = nullptr;
 			if (reset)
 			{
 				guiprof = GUIProfile::find(mixer, QString("default"), false, true); // ### Card unspecific profile ###
 			}
 
-			if ( guiprof != 0 )
+			if ( guiprof!=nullptr)
 			{
 				guiprof->setDirty();  // All fallback => dirty
 				addMixerWidget(mixer->id(), guiprof->getId(), -1);
@@ -665,13 +675,13 @@ void KMixWindow::recreateGUI(bool saveConfig, const QString& mixerId, bool force
 
 			// Lets try a bunch of fallback strategies:
 			qCDebug(KMIX_LOG) << "Attempting to find a card-specific GUI Profile for the mixer " << mixer->id();
-			GUIProfile* guiprof = GUIProfile::find(mixer, QString("default"), false, false);// ### Card specific profile ###
-			if ( guiprof == 0 )
+			GUIProfile *guiprof = GUIProfile::find(mixer, QString("default"), false, false);// ### Card specific profile ###
+			if (guiprof==nullptr)
 			{
 				qCDebug(KMIX_LOG) << "Not found. Attempting to find a generic GUI Profile for the mixer " << mixer->id();
 				guiprof = GUIProfile::find(mixer, QString("default"), false, true); // ### Card unspecific profile ###
 			}
-			if ( guiprof == 0)
+			if (guiprof==nullptr)
 			{
 				qCDebug(KMIX_LOG) << "Using fallback GUI Profile for the mixer " << mixer->id();
 				// This means there is neither card specific nor card unspecific profile
@@ -679,7 +689,7 @@ void KMixWindow::recreateGUI(bool saveConfig, const QString& mixerId, bool force
 				guiprof = GUIProfile::fallbackProfile(mixer);
 			}
 
-			if ( guiprof != 0 )
+			if (guiprof!=nullptr)
 			{
 				guiprof->setDirty();  // All fallback => dirty
 				addMixerWidget(mixer->id(), guiprof->getId(), -1);
@@ -694,39 +704,50 @@ void KMixWindow::recreateGUI(bool saveConfig, const QString& mixerId, bool force
 	mixerHasProfile.clear();
 
 	// -4- FINALIZE **********************************
-	if (m_wsMixers->count() > 0)
+
+
+	// Show the system tray icon, if it is enabled and there
+	// is at least one sound card.
+	const bool dockingSucceded = updateDocking();
+	if (reset)
 	{
-		if (oldTabPosition >= 0)
-		{
-			m_wsMixers->setCurrentIndex(oldTabPosition);
-		}
-		bool dockingSucceded = updateDocking();
-		if (!dockingSucceded && !MixerToolBox::mixers().empty())
-		{
-			show(); // avoid invisible and inaccessible main window
-		}
+		// Always show the main window, along with the system tray
+		// icon if enabled, on a GUI reset.
+		show();
+	}
+	else if (m_wsMixers->count()>0)			// at least one tab present
+	{
+		if (oldTabPosition>=0) m_wsMixers->setCurrentIndex(oldTabPosition);
+
+		// If there is no system tray icon, then ensure that the
+		// main window is shown.  Originally this also checked
+		// for '!MixerToolBox::mixers().empty()', but if that is
+		// the case then there should be at least one tab.
+		if (!dockingSucceded) show();
 	}
 	else
 	{
-		// No soundcard found. Do not complain, but sit in the background, and wait for newly plugged soundcards.
-		updateDocking();  // -<- removes the DockIcon
+		// TODO: is this correct?  This means that there is no GUI
+		// way to quit KMix if there are no sound cards.
+		//
+		// No sound card was found.  Do not complain, but just sit
+		// in the background and wait for newly plugged soundcards.
+		// The updateDocking() below will also remove the system
+		// tray icon until there is a sound card available.
+		updateDocking();
 		hide();
 	}
-
 }
 
-KMixerWidget*
-KMixWindow::findKMWforTab(const QString& kmwId)
+
+KMixerWidget *KMixWindow::findKMWforTab(const QString &kmwId)
 {
 	for (int i = 0; i < m_wsMixers->count(); ++i)
 	{
 		KMixerWidget *kmw = qobject_cast<KMixerWidget *>(m_wsMixers->widget(i));
-		if (kmw->getGuiprof()->getId() == kmwId)
-		{
-			return kmw;
-		}
+		if (kmw->getGuiprof()->getId() == kmwId) return (kmw);
 	}
-	return 0;
+	return (nullptr);
 }
 
 void KMixWindow::newView()
@@ -739,15 +760,13 @@ void KMixWindow::newView()
 	}
 
 	Mixer *mixer = mixers.first();
-	QPointer<DialogAddView> dav = new DialogAddView(this, mixer);
-	int ret = dav->exec();
+	DialogAddView dav(this, mixer);
 
-	// TODO: it is pointless using a smart pointer for the dialogue
-	// (which is good practice) here and then not checking it!
-	if (QDialog::Accepted == ret)
+	const int ret = dav.exec();
+	if (ret==QDialog::Accepted)
 	{
-		QString profileName = dav->getresultViewName();
-		QString mixerId = dav->getresultMixerId();
+		QString profileName = dav.getresultViewName();
+		QString mixerId = dav.getresultMixerId();
 		mixer = MixerToolBox::findMixer(mixerId);
 		qCDebug(KMIX_LOG) << ">>> mixer = " << mixerId << " -> " << mixer;
 
@@ -763,17 +782,13 @@ void KMixWindow::newView()
 		}
 		else
 		{
-			bool ret = addMixerWidget(mixer->id(), guiprof->getId(), -1);
-			if (!ret)
+			const bool added = addMixerWidget(mixer->id(), guiprof->getId(), -1);
+			if (!added)
 			{
 				KMessageBox::error(this, i18n("Cannot add view - View already exists."), i18n("Error"));
 			}
 		}
-
-		delete dav;
 	}
-
-	//qCDebug(KMIX_LOG) << "Exit";
 }
 
 /**
@@ -786,8 +801,8 @@ void KMixWindow::saveAndCloseView(int idx)
 	qCDebug(KMIX_LOG)
 	<< "Enter";
 	QWidget *w = m_wsMixers->widget(idx);
-	KMixerWidget* kmw = ::qobject_cast<KMixerWidget*>(w);
-	if (kmw)
+	KMixerWidget *kmw = qobject_cast<KMixerWidget *>(w);
+	if (kmw!=nullptr)
 	{
 		kmw->saveConfig(Settings::self()->config()); // -<- This alone is not enough, as I need to save the META information as well. Thus use saveViewConfig() below
 		m_wsMixers->removeTab(idx);
@@ -795,8 +810,8 @@ void KMixWindow::saveAndCloseView(int idx)
 		saveViewConfig();
 		delete kmw;
 	}
-	qCDebug(KMIX_LOG)
-	<< "Exit";
+
+	qCDebug(KMIX_LOG) << "Exit";
 }
 
 void KMixWindow::fixConfigAfterRead()
@@ -852,7 +867,7 @@ void KMixWindow::unplugged(const QString &udi)
 	// This assumes that there can be at most one mixer in the list
 	// with the given UDI.
 	Mixer *unpluggedMixer = nullptr;
-	for (Mixer *mixer : qAsConst(MixerToolBox::mixers()))
+	for (Mixer *mixer : std::as_const(MixerToolBox::mixers()))
 	{
 		if (mixer->hotplugId()==udi)
 		{
@@ -878,7 +893,7 @@ void KMixWindow::unplugged(const QString &udi)
 	// the given UDI, there can be more than one GUI tab for it.
 	for (int i = 0; i<m_wsMixers->count(); ++i)
 	{
-		KMixerWidget *kmw = qobject_cast<KMixerWidget *>(m_wsMixers->widget(i));
+		const KMixerWidget *kmw = qobject_cast<const KMixerWidget *>(m_wsMixers->widget(i));
 		if (kmw!=nullptr && kmw->mixer()==unpluggedMixer)
 		{
 			saveAndCloseView(i);
@@ -941,11 +956,10 @@ bool KMixWindow::profileExists(QString guiProfileId)
 {
 	for (int i = 0; i < m_wsMixers->count(); ++i)
 	{
-		KMixerWidget* kmw = dynamic_cast<KMixerWidget*>(m_wsMixers->widget(i));
-		if (kmw && kmw->getGuiprof()->getId() == guiProfileId)
-			return true;
+		const KMixerWidget *kmw = qobject_cast<KMixerWidget *>(m_wsMixers->widget(i));
+		if (kmw!=nullptr && kmw->getGuiprof()->getId()==guiProfileId) return (true);
 	}
-	return false;
+	return (false);
 }
 
 bool KMixWindow::addMixerWidget(const QString& mixer_ID, QString guiprofId, int insertPosition)
@@ -953,14 +967,14 @@ bool KMixWindow::addMixerWidget(const QString& mixer_ID, QString guiprofId, int
 	qCDebug(KMIX_LOG)
 	<< "Add " << guiprofId;
 	GUIProfile* guiprof = GUIProfile::find(guiprofId);
-	if (guiprof != 0 && profileExists(guiprof->getId())) // TODO Bad place. Should be checked in the add-tab-dialog
-		return false; // already present => don't add again
+	if (guiprof!=nullptr && profileExists(guiprof->getId())) // TODO Bad place. Should be checked in the add-tab-dialog
+		return (false); // already present => don't add again
 	Mixer *mixer = MixerToolBox::findMixer(mixer_ID);
 	if (mixer==nullptr) return (false);		// no such Mixer
 
 	//       qCDebug(KMIX_LOG) << "KMixWindow::addMixerWidget() " << mixer_ID << " is being added";
 	ViewBase::ViewFlags vflags = ViewBase::HasMenuBar;
-	if ((_actionShowMenubar == 0) || _actionShowMenubar->isChecked())
+	if ((_actionShowMenubar==nullptr) || _actionShowMenubar->isChecked())
 		vflags |= ViewBase::MenuBarVisible;
 	KMixerWidget *kmw = new KMixerWidget(mixer, this, vflags, guiprofId, actionCollection());
 	/* A newly added mixer will automatically added at the top
@@ -995,57 +1009,50 @@ bool KMixWindow::addMixerWidget(const QString& mixer_ID, QString guiprofId, int
 	// Now force to read for new tabs, especially after hotplug. Note: Doing it here is bad design and possibly
 	// obsolete, as the backend should take care of updating itself.
 	kmw->mixer()->readSetFromHWforceUpdate();
-	return true;
+
+	return (true);
 }
 
 void KMixWindow::updateTabsClosable()
 {
-	// Pulseaudio runs with 4 fixed tabs - don't allow to close them.
+	// PulseAudio runs with 4 fixed tabs - don't allow to close them.
 	// Also do not allow to close the last view
 	m_wsMixers->setTabsClosable(!MixerToolBox::pulseaudioPresent() && m_wsMixers->count() > 1);
 }
 
-bool KMixWindow::queryClose()
+
+void KMixWindow::closeEvent(QCloseEvent *ev)
 {
-	if (Settings::showDockWidget() && !qApp->isSavingSession())
+	if (m_dockWidget!=nullptr && !qApp->isSavingSession())
 	{
-		// Hide (don't close and destroy), if docking is enabled. Except when session saving (shutdown) is in process.
+		// The system tray icon is present, so just hide the
+		// main window and do not close it.  The test for
+		// whether m_dockWidget exists will have also taken
+		// account of shouldShowDock().
 		hide();
-		return false;
+		ev->ignore();
 	}
 	else
 	{
-		// Accept the close, if:
-		//     The user has disabled docking
-		// or  SessionSaving() is running
-		//         qCDebug(KMIX_LOG) << "close";
-		return true;
+		// There is no system tray icon, so closing this
+		// window quits the application.  Because we cannot
+		// set WA_DeleteOnClose for this KMixWindow (because
+		// KMixApp just keeps a simple pointer to its m_kmix
+		// and so cannot know when it gets deleted), it is
+		// necessary to explicitly quit the application here.
+		QCoreApplication::quit();
+		ev->accept();
 	}
 }
 
-void KMixWindow::hideOrClose()
-{
-	if (Settings::showDockWidget() && m_dockWidget!=nullptr)
-	{
-		// we can hide if there is a dock widget
-		hide();
-	}
-	else
-	{
-		//  if there is no dock widget, we will quit
-		quit();
-	}
-}
 
 // internal helper to prevent code duplication in slotIncreaseVolume and slotDecreaseVolume
 void KMixWindow::increaseOrDecreaseVolume(bool increase)
 {
 	Mixer* mixer = MixerToolBox::getGlobalMasterMixer(); // only needed for the awkward construct below
-	if (mixer == 0)
-		return; // e.g. when no soundcard is available
+	if (mixer==nullptr) return;			     // e.g. when no soundcard is available
 	shared_ptr<MixDevice> md = MixerToolBox::getGlobalMasterMD();
-	if (md.get() == 0)
-		return; // shouldn't happen, but lets play safe
+	if (md.get()==nullptr) return;			// shouldn't happen, but lets play safe
 
 	Volume::VolumeTypeFlag volumeType = md->playbackVolume().hasVolume() ? Volume::Playback : Volume::Capture;
 	md->increaseOrDecreaseVolume(!increase, volumeType);
@@ -1067,11 +1074,9 @@ void KMixWindow::slotDecreaseVolume()
 void KMixWindow::showVolumeDisplay()
 {
 	Mixer* mixer = MixerToolBox::getGlobalMasterMixer();
-	if (mixer == 0)
-		return; // e.g. when no soundcard is available
+	if (mixer==nullptr) return;			// e.g. when no soundcard is available
 	shared_ptr<MixDevice> md = MixerToolBox::getGlobalMasterMD();
-	if (md.get() == 0)
-		return; // shouldn't happen, but lets play safe
+	if (md.get()==nullptr) return;			// shouldn't happen, but lets play safe
 
 	if (Settings::showOSD())
 	{
@@ -1094,27 +1099,19 @@ void KMixWindow::showVolumeDisplay()
 }
 
 /**
- * Mutes the global master. (SLOT)
+ * Mutes the global master.
  */
 void KMixWindow::slotMute()
 {
 	Mixer* mixer = MixerToolBox::getGlobalMasterMixer();
-	if (mixer == 0)
-		return; // e.g. when no soundcard is available
+	if (mixer==nullptr) return;			// e.g. when no soundcard is available
 	shared_ptr<MixDevice> md = MixerToolBox::getGlobalMasterMD();
-	if (md.get() == 0)
-		return; // shouldn't happen, but lets play safe
+	if (md.get()==nullptr) return;			// shouldn't happen, but lets play safe
 	md->toggleMute();
 	mixer->commitVolumeChange(md);
 	showVolumeDisplay();
 }
 
-void KMixWindow::quit()
-{
-	//     qCDebug(KMIX_LOG) << "quit";
-	qApp->quit();
-}
-
 /**
  * Shows the configuration dialog, with the "General" tab opened.
  */
@@ -1174,11 +1171,13 @@ void KMixWindow::toggleMenuBar()
 	menuBar()->setVisible(_actionShowMenubar->isChecked());
 }
 
+
 void KMixWindow::slotKdeAudioSetupExec()
 {
-    forkExec(QStringList() << "kcmshell5" << "kcm_pulseaudio");
+    forkExec(QStringList() << QString("kcmshell%1").arg(KCOREADDONS_VERSION_MAJOR) << "kcm_pulseaudio");
 }
 
+
 void KMixWindow::forkExec(const QStringList& args)
 {
    int pid = KProcess::startDetached(args);
@@ -1191,33 +1190,31 @@ void KMixWindow::forkExec(const QStringList& args)
 
 void KMixWindow::slotConfigureCurrentView()
 {
-	KMixerWidget *mw = qobject_cast<KMixerWidget *>(m_wsMixers->currentWidget());
-	ViewBase* view = 0;
-	if (mw)
-		view = mw->currentView();
-	if (view)
-		view->configureView();
+	const KMixerWidget *mw = qobject_cast<const KMixerWidget  *>(m_wsMixers->currentWidget());
+	if (mw==nullptr) return;
+	ViewBase *view = mw->currentView();
+	if (view!=nullptr) view->configureView();
 }
 
-void KMixWindow::slotSelectMasterClose(QObject*)
-{
-	m_dsm = 0;
-}
 
 void KMixWindow::slotSelectMaster()
 {
 	const Mixer *mixer = MixerToolBox::getGlobalMasterMixer();
 	if (mixer!=nullptr)
 	{
-		if (!m_dsm)
+		// m_masterSelectDialog will probably always be NULL here,
+		// because closing the dialogue deletes itself (because of
+		// the WA_DeleteOnClose set below) and the QPointer tracks
+		// that deletion.
+		if (m_masterSelectDialog.isNull())
 		{
-			m_dsm = new DialogSelectMaster(mixer, this);
-			connect(m_dsm, SIGNAL(destroyed(QObject*)), this, SLOT(slotSelectMasterClose(QObject*)));
-			m_dsm->setAttribute(Qt::WA_DeleteOnClose, true);
-			m_dsm->show();
+			m_masterSelectDialog = new DialogSelectMaster(mixer, this);
+			m_masterSelectDialog->setAttribute(Qt::WA_DeleteOnClose, true);
+			m_masterSelectDialog->show();
 		}
-		m_dsm->raise();
-		m_dsm->activateWindow();
+
+		m_masterSelectDialog->raise();
+		m_masterSelectDialog->activateWindow();
 	}
 	else
 	{
@@ -1225,26 +1222,42 @@ void KMixWindow::slotSelectMaster()
 	}
 }
 
+
 void KMixWindow::newMixerShown(int /*tabIndex*/)
 {
-	KMixerWidget *kmw = qobject_cast<KMixerWidget *>(m_wsMixers->currentWidget());
-	if (kmw!=nullptr)
-	{
-		// I am using the app name as a PREFIX, as KMix is a single window app, and it is
-		// more helpful to the user to see "KDE Mixer" in a window list than a possibly cryptic
-		// soundcard name like "HDA ATI SB".
-		// Reformatted for KF5 so as to not say "KDE"
-		// and so that there are not two different dashes.
-		setWindowTitle(i18n("Mixer (%1)", kmw->mixer()->readableName()));
-		if (!m_dontSetDefaultCardOnStart)
-			m_defaultCardOnStart = kmw->getGuiprof()->getId();
-		// As switching the tab does NOT mean switching the master card, we do not need to update dock icon here.
-		// It would lead to unnecesary flickering of the (complete) dock area.
+	const KMixerWidget *kmw = qobject_cast<const KMixerWidget *>(m_wsMixers->currentWidget());
+	if (kmw==nullptr) return;			// no current mixer
 
-		// We only show the "Configure Channels..." menu item if the mixer is not dynamic
-		ViewBase* view = kmw->currentView();
-		QAction* action = actionCollection()->action("toggle_channels_currentview");
-		if (view && action)
-			action->setVisible(!view->isDynamic());
+	// I am using the app name as a PREFIX, as KMix is a single window
+	// application and it is more helpful to the user to see "KDE Mixer"
+	// in a window list than a possibly cryptic soundcard name like "HDA ATI SB".
+	// Reformatted for KF5 so as to not say "KDE" and so that there are not
+	// two different dashes.
+	setWindowTitle(i18n("Mixer (%1)", kmw->mixer()->readableName()));
+
+	if (!m_dontSetDefaultCardOnStart)
+		m_defaultCardOnStart = kmw->getGuiprof()->getId();
+
+	// As switching the tab does NOT mean switching the master card,
+	// we do not need to update the dock icon here.  It would lead to
+	// unnecesary flickering of the (complete) dock area.
+
+	// The "Configure Channels..." action is only applicable if the
+	// current mixer is not dynamic.  In order to keep the presence
+	// or absence of the action consistent between tabs, if the current
+	// mixer is PulseAudio then the action is hidden because it will
+	// never be applicable.  Otherwise the action is shown, and enabled
+	// unless the current mixer is dynamic - where the only other dynamic
+	// mixer is MPRIS2.
+	const ViewBase *view = kmw->currentView();
+	QAction *action = actionCollection()->action("toggle_channels_currentview");
+	if (view!=nullptr && action!=nullptr)
+	{
+		if (kmw->mixer()->getDriverName()=="PulseAudio") action->setVisible(false);
+		else
+		{
+			action->setVisible(true);
+			action->setEnabled(!view->isDynamic());
+		}
 	}
 }
