@@ -47,7 +47,7 @@ KMixerWidget::KMixerWidget(Mixer *mixer,
                             QWidget * parent, ViewBase::ViewFlags vflags, const QString &guiprofId,
                             KActionCollection* actionCollection )
    : QWidget( parent ), _mixer(mixer),
-     m_topLayout(0), _guiprofId(guiprofId),
+     m_topLayout(nullptr), _guiprofId(guiprofId),
      _actionCollection(actionCollection)
 {
 	createLayout(vflags);
@@ -79,8 +79,8 @@ void KMixerWidget::createLayout(ViewBase::ViewFlags vflags)
    * 2b) Create device widgets
    * 2c) Add Views to Tab
    ********************************************************************/
-   GUIProfile* guiprof = getGuiprof();
-   if ( guiprof != 0 )
+   GUIProfile *guiprof = getGuiprof();
+   if (guiprof!=nullptr)
    {
        if (Settings::debugGui())
 		qCDebug(KMIX_LOG) << "Add a view " << _guiprofId;
@@ -91,37 +91,37 @@ void KMixerWidget::createLayout(ViewBase::ViewFlags vflags)
 }
 
 
-
 /**
  * Add the given view, if it is valid - it must have controls or at least have the chance to gain some (dynamic views)
  * @param vbase
  * @return true, if the view was added
  */
-bool KMixerWidget::possiblyAddView(ViewBase* vbase)
+bool KMixerWidget::possiblyAddView(ViewBase *vbase)
 {
-   if ( ! vbase->isValid()  ) {
-      delete vbase;
-      return false;
-   }
-   else {
-      m_topLayout->addWidget(vbase);
-      _views.push_back(vbase);
-      connect( vbase, SIGNAL(toggleMenuBar()), parentWidget(), SLOT(toggleMenuBar()) );
-      if (Settings::debugGui())
-    	  qCDebug(KMIX_LOG) << "CONNECT ViewBase count " << vbase->getMixers().size();
-      return true;
+    if (!vbase->isValid())
+    {
+        delete vbase;
+        return (false);
    }
+
+    m_topLayout->addWidget(vbase);
+    _views.push_back(vbase);
+
+    if (Settings::debugGui())
+        qCDebug(KMIX_LOG) << "CONNECT ViewBase count " << vbase->getMixers().size();
+    return (true);
 }
 
+
 /**
  * Returns the current View. Normally we have only one View, so we always return the first view.
  * This method is only here for one reason: We can plug in an action in the main menu for the view, so that
  * 99,99% of all users will be well served. Those who hack their own XML Profile to contain more than one view
   must  use the context menu for configuring the additional views.
  */
-ViewBase* KMixerWidget::currentView()
+ViewBase *KMixerWidget::currentView() const
 {
-	return _views.empty() ? 0 : _views[0];
+    return (_views.empty() ? nullptr : _views[0]);
 }
 
 
@@ -161,9 +161,3 @@ void KMixerWidget::saveConfig(KConfig *config)
 		view->save(config);
 	} // for all tabs
 }
-
-
-void KMixerWidget::toggleMenuBarSlot() {
-    emit toggleMenuBar();
-}
-
